"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
import pydantic
from pydantic import model_serializer
from squadcast_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V3IncidentsLogsAdditionalInfoTypedDict(TypedDict):
    pass


class V3IncidentsLogsAdditionalInfo(BaseModel):
    pass


class V3IncidentsLogsTypedDict(TypedDict):
    r"""Represents a log entry for an incident action."""

    action: str
    time: datetime
    reason: str
    is_manually_created: NotRequired[bool]
    created_by: NotRequired[str]
    updated_by: NotRequired[str]
    assigned_to: NotRequired[str]
    assigned_by: NotRequired[str]
    id: NotRequired[str]
    workflow_id: NotRequired[int]
    additional_info: NotRequired[Nullable[V3IncidentsLogsAdditionalInfoTypedDict]]
    type: NotRequired[str]
    should_show_in_postmortem: NotRequired[bool]


class V3IncidentsLogs(BaseModel):
    r"""Represents a log entry for an incident action."""

    action: str

    time: datetime

    reason: str

    is_manually_created: Optional[bool] = None

    created_by: Optional[str] = None

    updated_by: Optional[str] = None

    assigned_to: Annotated[Optional[str], pydantic.Field(alias="assignedTo")] = None

    assigned_by: Annotated[Optional[str], pydantic.Field(alias="assignedBy")] = None

    id: Optional[str] = None

    workflow_id: Optional[int] = None

    additional_info: Annotated[
        OptionalNullable[V3IncidentsLogsAdditionalInfo],
        pydantic.Field(alias="additionalInfo"),
    ] = UNSET

    type: Optional[str] = None

    should_show_in_postmortem: Optional[bool] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "is_manually_created",
            "created_by",
            "updated_by",
            "assignedTo",
            "assignedBy",
            "id",
            "workflow_id",
            "additionalInfo",
            "type",
            "should_show_in_postmortem",
        ]
        nullable_fields = ["additionalInfo"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_escalationpolicies_escalationentity import (
    V3EscalationPoliciesEscalationEntity,
    V3EscalationPoliciesEscalationEntityTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import Annotated, TypedDict


class V3EscalationPoliciesEscalationPolicyRuleTypedDict(TypedDict):
    r"""Represents a rule within an escalation policy."""

    escalation_time: int
    r"""The time in minutes to wait before this rule is triggered."""
    via: List[str]
    r"""The notification methods to use for this rule."""
    roundrobin_enabled: bool
    r"""Indicates if round-robin is enabled for the entities in this rule."""
    roundrobin_next_index: int
    r"""The index of the next entity to be notified in a round-robin setup."""
    entities: List[V3EscalationPoliciesEscalationEntityTypedDict]
    r"""The entities to be notified in this rule."""
    escalate_within_roundrobin: bool
    r"""Indicates if escalation should happen within the round-robin rotation."""
    repetition: int
    r"""The number of times this specific rule should be repeated."""
    repeat_after: int
    r"""The time in minutes after which this rule should be repeated."""


class V3EscalationPoliciesEscalationPolicyRule(BaseModel):
    r"""Represents a rule within an escalation policy."""

    escalation_time: Annotated[int, pydantic.Field(alias="escalationTime")]
    r"""The time in minutes to wait before this rule is triggered."""

    via: List[str]
    r"""The notification methods to use for this rule."""

    roundrobin_enabled: bool
    r"""Indicates if round-robin is enabled for the entities in this rule."""

    roundrobin_next_index: int
    r"""The index of the next entity to be notified in a round-robin setup."""

    entities: List[V3EscalationPoliciesEscalationEntity]
    r"""The entities to be notified in this rule."""

    escalate_within_roundrobin: bool
    r"""Indicates if escalation should happen within the round-robin rotation."""

    repetition: int
    r"""The number of times this specific rule should be repeated."""

    repeat_after: int
    r"""The time in minutes after which this rule should be repeated."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


V3WorkflowsSqAddIncidentNoteName = Literal["sq_add_incident_note",]


class V3WorkflowsSqAddIncidentNoteDataTypedDict(TypedDict):
    note: str


class V3WorkflowsSqAddIncidentNoteData(BaseModel):
    note: str


class V3WorkflowsSqAddIncidentNoteTypedDict(TypedDict):
    name: V3WorkflowsSqAddIncidentNoteName
    data: V3WorkflowsSqAddIncidentNoteDataTypedDict


class V3WorkflowsSqAddIncidentNote(BaseModel):
    name: V3WorkflowsSqAddIncidentNoteName

    data: V3WorkflowsSqAddIncidentNoteData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_overlay_overlayresponse import (
    V3ServicesOverlayOverlayResponse,
    V3ServicesOverlayOverlayResponseTypedDict,
)
from .v3_services_overlay_updatededupkeyoverlayrequest import (
    V3ServicesOverlayUpdateDedupKeyOverlayRequest,
    V3ServicesOverlayUpdateDedupKeyOverlayRequestTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class OverlayUpdateDedupKeyOverlayRequestTypedDict(TypedDict):
    service_id: str
    alert_source: str
    v3_services_overlay_update_dedup_key_overlay_request: (
        V3ServicesOverlayUpdateDedupKeyOverlayRequestTypedDict
    )


class OverlayUpdateDedupKeyOverlayRequest(BaseModel):
    service_id: Annotated[
        str,
        pydantic.Field(alias="serviceID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    alert_source: Annotated[
        str,
        pydantic.Field(alias="alertSource"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_services_overlay_update_dedup_key_overlay_request: Annotated[
        V3ServicesOverlayUpdateDedupKeyOverlayRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class ResponseBody2TypedDict(TypedDict):
    data: V3ServicesOverlayOverlayResponseTypedDict


class ResponseBody2(BaseModel):
    data: V3ServicesOverlayOverlayResponse


class ResponseBody1TypedDict(TypedDict):
    data: V3ServicesOverlayOverlayResponseTypedDict


class ResponseBody1(BaseModel):
    data: V3ServicesOverlayOverlayResponse


OverlayUpdateDedupKeyOverlayResponseTypedDict = TypeAliasType(
    "OverlayUpdateDedupKeyOverlayResponseTypedDict",
    Union[ResponseBody1TypedDict, ResponseBody2TypedDict],
)
r"""The request has succeeded."""


OverlayUpdateDedupKeyOverlayResponse = TypeAliasType(
    "OverlayUpdateDedupKeyOverlayResponse", Union[ResponseBody1, ResponseBody2]
)
r"""The request has succeeded."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


V3ServicesDedupInitConfigTimeUnit = Literal[
    "minute",
    "hour",
]


class V3ServicesDedupInitConfigTypedDict(TypedDict):
    time_window: int
    time_unit: V3ServicesDedupInitConfigTimeUnit


class V3ServicesDedupInitConfig(BaseModel):
    time_window: int

    time_unit: V3ServicesDedupInitConfigTimeUnit



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3WorkflowsSqAddIncidentNoteUpdateName = Literal["sq_add_incident_note",]


class V3WorkflowsSqAddIncidentNoteUpdateDataTypedDict(TypedDict):
    note: NotRequired[str]


class V3WorkflowsSqAddIncidentNoteUpdateData(BaseModel):
    note: Optional[str] = None


class V3WorkflowsSqAddIncidentNoteUpdateTypedDict(TypedDict):
    name: V3WorkflowsSqAddIncidentNoteUpdateName
    data: NotRequired[V3WorkflowsSqAddIncidentNoteUpdateDataTypedDict]


class V3WorkflowsSqAddIncidentNoteUpdate(BaseModel):
    name: V3WorkflowsSqAddIncidentNoteUpdateName

    data: Optional[V3WorkflowsSqAddIncidentNoteUpdateData] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
import httpx
import importlib
from squadcast_sdk import models, utils
from squadcast_sdk._hooks import SDKHooks
from squadcast_sdk.types import OptionalNullable, UNSET
import sys
from typing import Any, Callable, Dict, Optional, TYPE_CHECKING, Union, cast
import weakref

if TYPE_CHECKING:
    from squadcast_sdk.additionalresponders import AdditionalResponders
    from squadcast_sdk.analytics import Analytics
    from squadcast_sdk.auditlogs import AuditLogs
    from squadcast_sdk.communicationcards import CommunicationCards
    from squadcast_sdk.componentgroups import ComponentGroups
    from squadcast_sdk.components import Components
    from squadcast_sdk.dedupkeys import DedupKeys
    from squadcast_sdk.escalationpolicies_sdk import EscalationPoliciesSDK
    from squadcast_sdk.exports import Exports
    from squadcast_sdk.exportschedule import ExportSchedule
    from squadcast_sdk.extensions import Extensions
    from squadcast_sdk.globaleventrules import GlobalEventRules
    from squadcast_sdk.globaloncallreminderrules_sdk import GlobalOncallReminderRulesSDK
    from squadcast_sdk.incidents import Incidents
    from squadcast_sdk.issues import Issues
    from squadcast_sdk.maintenances import Maintenances
    from squadcast_sdk.overlays import Overlays
    from squadcast_sdk.overrides import Overrides
    from squadcast_sdk.postmortems_sdk import PostmortemsSDK
    from squadcast_sdk.rotations import Rotations
    from squadcast_sdk.rules import Rules
    from squadcast_sdk.rulesets import Rulesets
    from squadcast_sdk.runbooks_sdk import RunbooksSDK
    from squadcast_sdk.schedules_sdk import SchedulesSDK
    from squadcast_sdk.services_sdk import ServicesSDK
    from squadcast_sdk.slos_sdk import SlosSDK
    from squadcast_sdk.snoozenotifications import SnoozeNotifications
    from squadcast_sdk.squads_sdk import SquadsSDK
    from squadcast_sdk.squadsv4 import SquadsV4
    from squadcast_sdk.statuspages_sdk_1 import StatusPagesSDK1
    from squadcast_sdk.statuspages_sdk_2 import StatuspagesSDK2
    from squadcast_sdk.subscribers import Subscribers
    from squadcast_sdk.teams import Teams
    from squadcast_sdk.users import Users
    from squadcast_sdk.webforms_sdk import WebformsSDK
    from squadcast_sdk.webhooks import Webhooks
    from squadcast_sdk.workflows_sdk import WorkflowsSDK


class SquadcastSDK(BaseSDK):
    analytics: "Analytics"
    audit_logs: "AuditLogs"
    escalation_policies: "EscalationPoliciesSDK"
    exports: "Exports"
    extensions: "Extensions"
    webhooks: "Webhooks"
    incidents: "Incidents"
    global_event_rules: "GlobalEventRules"
    rulesets: "Rulesets"
    rules: "Rules"
    global_oncall_reminder_rules: "GlobalOncallReminderRulesSDK"
    postmortems: "PostmortemsSDK"
    communication_cards: "CommunicationCards"
    runbooks: "RunbooksSDK"
    additional_responders: "AdditionalResponders"
    snooze_notifications: "SnoozeNotifications"
    users: "Users"
    services: "ServicesSDK"
    dedup_keys: "DedupKeys"
    overlays: "Overlays"
    slos: "SlosSDK"
    teams: "Teams"
    webforms: "WebformsSDK"
    workflows: "WorkflowsSDK"
    schedules: "SchedulesSDK"
    export_schedule: "ExportSchedule"
    overrides: "Overrides"
    rotations: "Rotations"
    squads: "SquadsSDK"
    squads_v4: "SquadsV4"
    status_pages: "StatusPagesSDK1"
    components: "Components"
    component_groups: "ComponentGroups"
    issues: "Issues"
    maintenances: "Maintenances"
    statuspages: "StatuspagesSDK2"
    subscribers: "Subscribers"
    _sub_sdk_map = {
        "analytics": ("squadcast_sdk.analytics", "Analytics"),
        "audit_logs": ("squadcast_sdk.auditlogs", "AuditLogs"),
        "escalation_policies": (
            "squadcast_sdk.escalationpolicies_sdk",
            "EscalationPoliciesSDK",
        ),
        "exports": ("squadcast_sdk.exports", "Exports"),
        "extensions": ("squadcast_sdk.extensions", "Extensions"),
        "webhooks": ("squadcast_sdk.webhooks", "Webhooks"),
        "incidents": ("squadcast_sdk.incidents", "Incidents"),
        "global_event_rules": ("squadcast_sdk.globaleventrules", "GlobalEventRules"),
        "rulesets": ("squadcast_sdk.rulesets", "Rulesets"),
        "rules": ("squadcast_sdk.rules", "Rules"),
        "global_oncall_reminder_rules": (
            "squadcast_sdk.globaloncallreminderrules_sdk",
            "GlobalOncallReminderRulesSDK",
        ),
        "postmortems": ("squadcast_sdk.postmortems_sdk", "PostmortemsSDK"),
        "communication_cards": (
            "squadcast_sdk.communicationcards",
            "CommunicationCards",
        ),
        "runbooks": ("squadcast_sdk.runbooks_sdk", "RunbooksSDK"),
        "additional_responders": (
            "squadcast_sdk.additionalresponders",
            "AdditionalResponders",
        ),
        "snooze_notifications": (
            "squadcast_sdk.snoozenotifications",
            "SnoozeNotifications",
        ),
        "users": ("squadcast_sdk.users", "Users"),
        "services": ("squadcast_sdk.services_sdk", "ServicesSDK"),
        "dedup_keys": ("squadcast_sdk.dedupkeys", "DedupKeys"),
        "overlays": ("squadcast_sdk.overlays", "Overlays"),
        "slos": ("squadcast_sdk.slos_sdk", "SlosSDK"),
        "teams": ("squadcast_sdk.teams", "Teams"),
        "webforms": ("squadcast_sdk.webforms_sdk", "WebformsSDK"),
        "workflows": ("squadcast_sdk.workflows_sdk", "WorkflowsSDK"),
        "schedules": ("squadcast_sdk.schedules_sdk", "SchedulesSDK"),
        "export_schedule": ("squadcast_sdk.exportschedule", "ExportSchedule"),
        "overrides": ("squadcast_sdk.overrides", "Overrides"),
        "rotations": ("squadcast_sdk.rotations", "Rotations"),
        "squads": ("squadcast_sdk.squads_sdk", "SquadsSDK"),
        "squads_v4": ("squadcast_sdk.squadsv4", "SquadsV4"),
        "status_pages": ("squadcast_sdk.statuspages_sdk_1", "StatusPagesSDK1"),
        "components": ("squadcast_sdk.components", "Components"),
        "component_groups": ("squadcast_sdk.componentgroups", "ComponentGroups"),
        "issues": ("squadcast_sdk.issues", "Issues"),
        "maintenances": ("squadcast_sdk.maintenances", "Maintenances"),
        "statuspages": ("squadcast_sdk.statuspages_sdk_2", "StatuspagesSDK2"),
        "subscribers": ("squadcast_sdk.subscribers", "Subscribers"),
    }

    def __init__(
        self,
        bearer_auth: Union[str, Callable[[], str]],
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param bearer_auth: The bearer_auth required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client(follow_redirects=True)
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient(follow_redirects=True)
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(bearer_auth):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: models.Security(bearer_auth=bearer_auth())
        else:
            security = models.Security(bearer_auth=bearer_auth)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
            parent_ref=self,
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        self.sdk_configuration = hooks.sdk_init(self.sdk_configuration)

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def dynamic_import(self, modname, retries=3):
        for attempt in range(retries):
            try:
                return importlib.import_module(modname)
            except KeyError:
                # Clear any half-initialized module and retry
                sys.modules.pop(modname, None)
                if attempt == retries - 1:
                    break
        raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = self.dynamic_import(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration, parent_ref=self)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_participant import V4Participant, V4ParticipantTypedDict
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V4ParticipantGroupTypedDict(TypedDict):
    participants: List[V4ParticipantTypedDict]


class V4ParticipantGroup(BaseModel):
    participants: List[V4Participant]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_issues_issuecomponent import (
    V4StatusPagesIssuesIssueComponent,
    V4StatusPagesIssuesIssueComponentTypedDict,
)
from .v4_statuspages_issues_issuestate import (
    V4StatusPagesIssuesIssueState,
    V4StatusPagesIssuesIssueStateTypedDict,
)
from datetime import datetime
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesIssuesIssueTypedDict(TypedDict):
    id: NotRequired[int]
    title: NotRequired[str]
    components: NotRequired[List[V4StatusPagesIssuesIssueComponentTypedDict]]
    current_state: NotRequired[V4StatusPagesIssuesIssueStateTypedDict]
    created_at: NotRequired[datetime]


class V4StatusPagesIssuesIssue(BaseModel):
    id: Optional[int] = None

    title: Optional[str] = None

    components: Optional[List[V4StatusPagesIssuesIssueComponent]] = None

    current_state: Annotated[
        Optional[V4StatusPagesIssuesIssueState], pydantic.Field(alias="currentState")
    ] = None

    created_at: Annotated[Optional[datetime], pydantic.Field(alias="createdAt")] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class SchedulesDeleteScheduleRequestTypedDict(TypedDict):
    schedule_id: str


class SchedulesDeleteScheduleRequest(BaseModel):
    schedule_id: Annotated[
        str,
        pydantic.Field(alias="scheduleID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]


class SchedulesDeleteScheduleResponseBodyTypedDict(TypedDict):
    r"""There is no content to send for this request, but the headers may be useful."""


class SchedulesDeleteScheduleResponseBody(BaseModel):
    r"""There is no content to send for this request, but the headers may be useful."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class CommonV3EntityOwnerTypedDict(TypedDict):
    r"""Represents the owner of an entity."""

    id: str
    r"""The ID of the owner."""
    type: str
    r"""The type of the owner ( \"user\", \"squad\")."""


class CommonV3EntityOwner(BaseModel):
    r"""Represents the owner of an entity."""

    id: str
    r"""The ID of the owner."""

    type: str
    r"""The type of the owner ( \"user\", \"squad\")."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_deduplicationrules_creatededuplicationrulesrequest import (
    V3ServicesDeduplicationRulesCreateDeduplicationRulesRequest,
    V3ServicesDeduplicationRulesCreateDeduplicationRulesRequestTypedDict,
)
from .v3_services_deduplicationrules_deduplicationruleresponse import (
    V3ServicesDeduplicationRulesDeduplicationRuleResponse,
    V3ServicesDeduplicationRulesDeduplicationRuleResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import List
from typing_extensions import Annotated, TypedDict


class DeduplicationRulesCreateOrUpdateDeduplicationRulesRequestTypedDict(TypedDict):
    service_id: str
    v3_services_deduplication_rules_create_deduplication_rules_request: (
        V3ServicesDeduplicationRulesCreateDeduplicationRulesRequestTypedDict
    )


class DeduplicationRulesCreateOrUpdateDeduplicationRulesRequest(BaseModel):
    service_id: Annotated[
        str,
        pydantic.Field(alias="serviceID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_services_deduplication_rules_create_deduplication_rules_request: Annotated[
        V3ServicesDeduplicationRulesCreateDeduplicationRulesRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class DeduplicationRulesCreateOrUpdateDeduplicationRulesDataTypedDict(TypedDict):
    rules: List[V3ServicesDeduplicationRulesDeduplicationRuleResponseTypedDict]


class DeduplicationRulesCreateOrUpdateDeduplicationRulesData(BaseModel):
    rules: List[V3ServicesDeduplicationRulesDeduplicationRuleResponse]


class DeduplicationRulesCreateOrUpdateDeduplicationRulesResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: DeduplicationRulesCreateOrUpdateDeduplicationRulesDataTypedDict


class DeduplicationRulesCreateOrUpdateDeduplicationRulesResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: DeduplicationRulesCreateOrUpdateDeduplicationRulesData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_issues_componentstatus import (
    V4StatusPagesIssuesComponentStatus,
    V4StatusPagesIssuesComponentStatusTypedDict,
)
from .v4_statuspages_issues_issuestatemessage import (
    V4StatusPagesIssuesIssueStateMessage,
    V4StatusPagesIssuesIssueStateMessageTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesIssuesIssueDetailTypedDict(TypedDict):
    state: NotRequired[V4StatusPagesIssuesComponentStatusTypedDict]
    state_messages: NotRequired[List[V4StatusPagesIssuesIssueStateMessageTypedDict]]


class V4StatusPagesIssuesIssueDetail(BaseModel):
    state: Optional[V4StatusPagesIssuesComponentStatus] = None

    state_messages: Annotated[
        Optional[List[V4StatusPagesIssuesIssueStateMessage]],
        pydantic.Field(alias="stateMessages"),
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsSnoozeNotificationsSnoozeIncidentRequestTypedDict(TypedDict):
    snooze_duration_in_mins: int


class V3IncidentsSnoozeNotificationsSnoozeIncidentRequest(BaseModel):
    snooze_duration_in_mins: int



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class V3AuditLogsActorTypedDict(TypedDict):
    r"""Represents an actor (user) in audit logs"""

    user_id: str
    user_name: str
    user_email: str
    full_name: str


class V3AuditLogsActor(BaseModel):
    r"""Represents an actor (user) in audit logs"""

    user_id: Annotated[str, pydantic.Field(alias="userID")]

    user_name: Annotated[str, pydantic.Field(alias="userName")]

    user_email: Annotated[str, pydantic.Field(alias="userEmail")]

    full_name: Annotated[str, pydantic.Field(alias="fullName")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_runbooks_attachrunbooksrequest import (
    V3IncidentsRunbooksAttachRunbooksRequest,
    V3IncidentsRunbooksAttachRunbooksRequestTypedDict,
)
from .v3_incidents_runbooks_runbookresponse import (
    V3IncidentsRunbooksRunbookResponse,
    V3IncidentsRunbooksRunbookResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import List
from typing_extensions import Annotated, TypedDict


class RunbooksAttachRunbooksRequestTypedDict(TypedDict):
    incident_id: str
    v3_incidents_runbooks_attach_runbooks_request: (
        V3IncidentsRunbooksAttachRunbooksRequestTypedDict
    )


class RunbooksAttachRunbooksRequest(BaseModel):
    incident_id: Annotated[
        str,
        pydantic.Field(alias="IncidentId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_incidents_runbooks_attach_runbooks_request: Annotated[
        V3IncidentsRunbooksAttachRunbooksRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class RunbooksAttachRunbooksResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: List[V3IncidentsRunbooksRunbookResponseTypedDict]


class RunbooksAttachRunbooksResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: List[V3IncidentsRunbooksRunbookResponse]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsPostmortemsTotalPostmortemCountTypedDict(TypedDict):
    r"""Represents the total count of postmortems."""

    count: int


class V3IncidentsPostmortemsTotalPostmortemCount(BaseModel):
    r"""Represents the total count of postmortems."""

    count: int



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_aptaconfig import V3ServicesAPTAConfig, V3ServicesAPTAConfigTypedDict
from .v3_services_iagconfig import V3ServicesIAGConfig, V3ServicesIAGConfigTypedDict
from .v3_services_notificationdelayconfigrequest import (
    V3ServicesNotificationDelayConfigRequest,
    V3ServicesNotificationDelayConfigRequestTypedDict,
)
from .v3_services_servicemaintainer import (
    V3ServicesServiceMaintainer,
    V3ServicesServiceMaintainerTypedDict,
)
from .v3_services_servicetag import V3ServicesServiceTag, V3ServicesServiceTagTypedDict
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3ServicesUpdateServiceRequestTypedDict(TypedDict):
    name: NotRequired[str]
    escalation_policy_id: NotRequired[str]
    description: NotRequired[str]
    email_prefix: NotRequired[str]
    maintainer: NotRequired[V3ServicesServiceMaintainerTypedDict]
    tags: NotRequired[List[V3ServicesServiceTagTypedDict]]
    auto_pause_transient_alerts_config: NotRequired[V3ServicesAPTAConfigTypedDict]
    intelligent_alerts_grouping_config: NotRequired[V3ServicesIAGConfigTypedDict]
    delay_notification_config: NotRequired[
        V3ServicesNotificationDelayConfigRequestTypedDict
    ]


class V3ServicesUpdateServiceRequest(BaseModel):
    name: Optional[str] = None

    escalation_policy_id: Optional[str] = None

    description: Optional[str] = None

    email_prefix: Optional[str] = None

    maintainer: Optional[V3ServicesServiceMaintainer] = None

    tags: Optional[List[V3ServicesServiceTag]] = None

    auto_pause_transient_alerts_config: Optional[V3ServicesAPTAConfig] = None

    intelligent_alerts_grouping_config: Optional[V3ServicesIAGConfig] = None

    delay_notification_config: Optional[V3ServicesNotificationDelayConfigRequest] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .common_v3_rbacentitypermission import (
    CommonV3RBACEntityPermission,
    CommonV3RBACEntityPermissionTypedDict,
)
from .common_v3_rbacowner import CommonV3RBACOwner, CommonV3RBACOwnerTypedDict
from .v3_incidents_postmortems_postmortemfollowup import (
    V3IncidentsPostmortemsPostmortemFollowUp,
    V3IncidentsPostmortemsPostmortemFollowUpTypedDict,
)
from .v3_incidents_postmortems_postmortemstatus import (
    V3IncidentsPostmortemsPostmortemStatus,
)
from pydantic import model_serializer
from squadcast_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3IncidentsPostmortemsPostmortemListResultIncidentTypedDict(TypedDict):
    message: str
    time_of_creation: str


class V3IncidentsPostmortemsPostmortemListResultIncident(BaseModel):
    message: str

    time_of_creation: str


class V3IncidentsPostmortemsPostmortemListResultTypedDict(TypedDict):
    r"""Represents a single result in the postmortem list response."""

    id: str
    organization_id: str
    incident_id: str
    postmortem: str
    follow_ups: Nullable[List[V3IncidentsPostmortemsPostmortemFollowUpTypedDict]]
    created_at: str
    title: str
    status: V3IncidentsPostmortemsPostmortemStatus
    r"""Represents the status of a postmortem."""
    owner: CommonV3RBACOwnerTypedDict
    r"""Represents the RBAC owner of an entity."""
    access_control: List[CommonV3RBACEntityPermissionTypedDict]
    incident_message: str
    incident_time_of_creation: str
    service_name: str
    service_id: str
    alert_source: str
    active_follow_ups_count: int
    attachments: NotRequired[Nullable[List[str]]]
    incident: NotRequired[V3IncidentsPostmortemsPostmortemListResultIncidentTypedDict]


class V3IncidentsPostmortemsPostmortemListResult(BaseModel):
    r"""Represents a single result in the postmortem list response."""

    id: str

    organization_id: str

    incident_id: str

    postmortem: str

    follow_ups: Nullable[List[V3IncidentsPostmortemsPostmortemFollowUp]]

    created_at: str

    title: str

    status: V3IncidentsPostmortemsPostmortemStatus
    r"""Represents the status of a postmortem."""

    owner: CommonV3RBACOwner
    r"""Represents the RBAC owner of an entity."""

    access_control: List[CommonV3RBACEntityPermission]

    incident_message: str

    incident_time_of_creation: str

    service_name: str

    service_id: str

    alert_source: str

    active_follow_ups_count: int

    attachments: OptionalNullable[List[str]] = UNSET

    incident: Optional[V3IncidentsPostmortemsPostmortemListResultIncident] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["attachments", "incident"]
        nullable_fields = ["follow_ups", "attachments"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3ExtensionsMSTeamsIncidentActionAlertStateTypedDict(TypedDict):
    r"""A set of booleans to easily configure which incident-related alerts are sent to MS Teams."""

    is_trigger_active: bool
    is_retrigger_active: bool
    is_acknowledge_active: bool
    is_resolve_active: bool
    is_reassign_active: bool
    is_notes_added_active: bool
    is_postmortem_active: bool
    is_priority_updated_active: bool


class V3ExtensionsMSTeamsIncidentActionAlertState(BaseModel):
    r"""A set of booleans to easily configure which incident-related alerts are sent to MS Teams."""

    is_trigger_active: bool

    is_retrigger_active: bool

    is_acknowledge_active: bool

    is_resolve_active: bool

    is_reassign_active: bool

    is_notes_added_active: bool

    is_postmortem_active: bool

    is_priority_updated_active: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


V3WorkflowsSlackArchiveChannelName = Literal["slack_archive_channel",]


class V3WorkflowsSlackArchiveChannelTypedDict(TypedDict):
    name: V3WorkflowsSlackArchiveChannelName


class V3WorkflowsSlackArchiveChannel(BaseModel):
    name: V3WorkflowsSlackArchiveChannelName



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


V3IncidentsIncidentExportRequestNotes = Literal[
    "present",
    "absent",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from squadcast_sdk.errors import SquadcastSDKError
from squadcast_sdk.models import (
    common_v3_errormeta as models_common_v3_errormeta,
    v3_incidents_incidentactions_circlecierrormeta as models_v3_incidents_incidentactions_circlecierrormeta,
)
from squadcast_sdk.types import BaseModel
from typing import Optional, Union
from typing_extensions import TypeAliasType


class GatewayTimeoutErrorData(BaseModel):
    meta: models_common_v3_errormeta.CommonV3ErrorMeta
    r"""Represents a single response containing data of type T."""


@dataclass(unsafe_hash=True)
class GatewayTimeoutError(SquadcastSDKError):
    r"""Server error"""

    data: GatewayTimeoutErrorData = field(hash=False)

    def __init__(
        self,
        data: GatewayTimeoutErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class ServiceUnavailableErrorData(BaseModel):
    meta: models_common_v3_errormeta.CommonV3ErrorMeta
    r"""Represents a single response containing data of type T."""


@dataclass(unsafe_hash=True)
class ServiceUnavailableError(SquadcastSDKError):
    r"""Service unavailable."""

    data: ServiceUnavailableErrorData = field(hash=False)

    def __init__(
        self,
        data: ServiceUnavailableErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class BadGatewayErrorData(BaseModel):
    meta: models_common_v3_errormeta.CommonV3ErrorMeta
    r"""Represents a single response containing data of type T."""


@dataclass(unsafe_hash=True)
class BadGatewayError(SquadcastSDKError):
    r"""Server error"""

    data: BadGatewayErrorData = field(hash=False)

    def __init__(
        self,
        data: BadGatewayErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class InternalServerErrorData(BaseModel):
    meta: models_common_v3_errormeta.CommonV3ErrorMeta
    r"""Represents a single response containing data of type T."""


@dataclass(unsafe_hash=True)
class InternalServerError(SquadcastSDKError):
    r"""Server error"""

    data: InternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: InternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class UnprocessableEntityErrorData(BaseModel):
    meta: models_common_v3_errormeta.CommonV3ErrorMeta
    r"""Represents a single response containing data of type T."""


@dataclass(unsafe_hash=True)
class UnprocessableEntityError(SquadcastSDKError):
    r"""Client error"""

    data: UnprocessableEntityErrorData = field(hash=False)

    def __init__(
        self,
        data: UnprocessableEntityErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class ConflictErrorData(BaseModel):
    meta: models_common_v3_errormeta.CommonV3ErrorMeta
    r"""Represents a single response containing data of type T."""


@dataclass(unsafe_hash=True)
class ConflictError(SquadcastSDKError):
    r"""The request conflicts with the current state of the server."""

    data: ConflictErrorData = field(hash=False)

    def __init__(
        self,
        data: ConflictErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class NotFoundErrorData(BaseModel):
    meta: models_common_v3_errormeta.CommonV3ErrorMeta
    r"""Represents a single response containing data of type T."""


@dataclass(unsafe_hash=True)
class NotFoundError(SquadcastSDKError):
    r"""The server cannot find the requested resource."""

    data: NotFoundErrorData = field(hash=False)

    def __init__(
        self,
        data: NotFoundErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class ForbiddenErrorData(BaseModel):
    meta: models_common_v3_errormeta.CommonV3ErrorMeta
    r"""Represents a single response containing data of type T."""


@dataclass(unsafe_hash=True)
class ForbiddenError(SquadcastSDKError):
    r"""Access is forbidden."""

    data: ForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: ForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class PaymentRequiredErrorData(BaseModel):
    meta: models_common_v3_errormeta.CommonV3ErrorMeta
    r"""Represents a single response containing data of type T."""


@dataclass(unsafe_hash=True)
class PaymentRequiredError(SquadcastSDKError):
    r"""Client error"""

    data: PaymentRequiredErrorData = field(hash=False)

    def __init__(
        self,
        data: PaymentRequiredErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class UnauthorizedErrorData(BaseModel):
    meta: models_common_v3_errormeta.CommonV3ErrorMeta
    r"""Represents a single response containing data of type T."""


@dataclass(unsafe_hash=True)
class UnauthorizedError(SquadcastSDKError):
    r"""Access is unauthorized."""

    data: UnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: UnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class BadRequestErrorData(BaseModel):
    meta: models_common_v3_errormeta.CommonV3ErrorMeta
    r"""Represents a single response containing data of type T."""


@dataclass(unsafe_hash=True)
class BadRequestError(SquadcastSDKError):
    r"""The server could not understand the request due to invalid syntax."""

    data: BadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: BadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class ResponseBodyError2Data(BaseModel):
    meta: models_common_v3_errormeta.CommonV3ErrorMeta
    r"""Represents a single response containing data of type T."""


@dataclass(unsafe_hash=True)
class ResponseBodyError2(SquadcastSDKError):
    data: ResponseBodyError2Data = field(hash=False)

    def __init__(
        self,
        data: ResponseBodyError2Data,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class ResponseBodyError1Data(BaseModel):
    meta: models_v3_incidents_incidentactions_circlecierrormeta.V3IncidentsIncidentActionsCircleCIErrorMeta
    r"""Represents the metadata for a CircleCI error response."""


@dataclass(unsafe_hash=True)
class ResponseBodyError1(SquadcastSDKError):
    data: ResponseBodyError1Data = field(hash=False)

    def __init__(
        self,
        data: ResponseBodyError1Data,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


BadRequestUnion = TypeAliasType(
    "BadRequestUnion", Union[ResponseBodyError1Data, ResponseBodyError2Data]
)
r"""Represents a CircleCI error response for a 400 status code."""


@dataclass(unsafe_hash=True)
class BadRequest(SquadcastSDKError):
    r"""Represents a CircleCI error response for a 400 status code."""

    data: BadRequestUnion = field(hash=False)

    def __init__(
        self,
        data: BadRequestUnion,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_extensions_msteams_channelconfiguration import (
    V3ExtensionsMSTeamsChannelConfiguration,
    V3ExtensionsMSTeamsChannelConfigurationTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3ExtensionsMSTeamsConnectedTeamsTypedDict(TypedDict):
    r"""Represents a connected MS Team and its channel configurations."""

    team_id: str
    r"""The unique identifier for the team in MS Teams."""
    team_name: str
    r"""The display name of the team in MS Teams."""
    channel_configurations: List[V3ExtensionsMSTeamsChannelConfigurationTypedDict]
    r"""A list of channel routing configurations for this team."""
    id: NotRequired[str]
    r"""The MongoDB ObjectID for this connected team entry."""


class V3ExtensionsMSTeamsConnectedTeams(BaseModel):
    r"""Represents a connected MS Team and its channel configurations."""

    team_id: str
    r"""The unique identifier for the team in MS Teams."""

    team_name: str
    r"""The display name of the team in MS Teams."""

    channel_configurations: List[V3ExtensionsMSTeamsChannelConfiguration]
    r"""A list of channel routing configurations for this team."""

    id: Optional[str] = None
    r"""The MongoDB ObjectID for this connected team entry."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .common_v4_pageinfo import CommonV4PageInfo, CommonV4PageInfoTypedDict
from .v4_squads_squadresponse import (
    V4SquadsSquadResponse,
    V4SquadsSquadResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import List
from typing_extensions import Annotated, TypedDict


class SquadsGetAllSquadsRequestTypedDict(TypedDict):
    owner_id: str


class SquadsGetAllSquadsRequest(BaseModel):
    owner_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]


class SquadsGetAllSquadsResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V4SquadsSquadResponseTypedDict]
    page_info: CommonV4PageInfoTypedDict


class SquadsGetAllSquadsResponse(BaseModel):
    r"""The request has succeeded."""

    data: List[V4SquadsSquadResponse]

    page_info: Annotated[CommonV4PageInfo, pydantic.Field(alias="pageInfo")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3WorkflowsMsTeamsMessageUserUpdateName = Literal["msteams_message_user",]


class V3WorkflowsMsTeamsMessageUserUpdateDataTypedDict(TypedDict):
    member_id: NotRequired[str]
    message: NotRequired[str]


class V3WorkflowsMsTeamsMessageUserUpdateData(BaseModel):
    member_id: Optional[str] = None

    message: Optional[str] = None


class V3WorkflowsMsTeamsMessageUserUpdateTypedDict(TypedDict):
    name: V3WorkflowsMsTeamsMessageUserUpdateName
    data: NotRequired[V3WorkflowsMsTeamsMessageUserUpdateDataTypedDict]


class V3WorkflowsMsTeamsMessageUserUpdate(BaseModel):
    name: V3WorkflowsMsTeamsMessageUserUpdateName

    data: Optional[V3WorkflowsMsTeamsMessageUserUpdateData] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_escalationpolicies_escalationpolicyresponse import (
    V3EscalationPoliciesEscalationPolicyResponse,
    V3EscalationPoliciesEscalationPolicyResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class EscalationPoliciesCreateEscalationPoliciesResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V3EscalationPoliciesEscalationPolicyResponseTypedDict
    r"""Represents an Escalation Policy in the system."""


class EscalationPoliciesCreateEscalationPoliciesResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V3EscalationPoliciesEscalationPolicyResponse
    r"""Represents an Escalation Policy in the system."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_workflows_communicationchannel import (
    V3WorkflowsCommunicationChannel,
    V3WorkflowsCommunicationChannelTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3WorkflowsSqAddCommunicationChannelUpdateName = Literal[
    "sq_add_communication_channel",
]


class V3WorkflowsSqAddCommunicationChannelUpdateDataTypedDict(TypedDict):
    channels: NotRequired[List[V3WorkflowsCommunicationChannelTypedDict]]


class V3WorkflowsSqAddCommunicationChannelUpdateData(BaseModel):
    channels: Optional[List[V3WorkflowsCommunicationChannel]] = None


class V3WorkflowsSqAddCommunicationChannelUpdateTypedDict(TypedDict):
    name: V3WorkflowsSqAddCommunicationChannelUpdateName
    data: NotRequired[V3WorkflowsSqAddCommunicationChannelUpdateDataTypedDict]


class V3WorkflowsSqAddCommunicationChannelUpdate(BaseModel):
    name: V3WorkflowsSqAddCommunicationChannelUpdateName

    data: Optional[V3WorkflowsSqAddCommunicationChannelUpdateData] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class V4StatusPagesComponentsDeleteComponentByIDResponseTypedDict(TypedDict):
    id: int
    name: str
    description: str
    allow_subscription: bool


class V4StatusPagesComponentsDeleteComponentByIDResponse(BaseModel):
    id: int

    name: str

    description: str

    allow_subscription: Annotated[bool, pydantic.Field(alias="allowSubscription")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .common_v3_entityowner import CommonV3EntityOwner, CommonV3EntityOwnerTypedDict
from datetime import datetime
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3RunbooksUpdationInfoTypedDict(TypedDict):
    r"""Represents information about the creation or updation of an entity."""

    user_name: str
    r"""The full name of the user who performed the action."""
    username_for_display: str
    r"""The display name of the user who performed the action."""
    user_id: str
    r"""The ID of the user who performed the action."""
    at: datetime
    r"""The timestamp of the action."""
    entity_owner: NotRequired[CommonV3EntityOwnerTypedDict]
    r"""The owner of the entity at the time of the action."""


class V3RunbooksUpdationInfo(BaseModel):
    r"""Represents information about the creation or updation of an entity."""

    user_name: str
    r"""The full name of the user who performed the action."""

    username_for_display: str
    r"""The display name of the user who performed the action."""

    user_id: str
    r"""The ID of the user who performed the action."""

    at: datetime
    r"""The timestamp of the action."""

    entity_owner: Optional[CommonV3EntityOwner] = None
    r"""The owner of the entity at the time of the action."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
import pydantic
from pydantic import model_serializer
from squadcast_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesMaintenancesGetMaintenanceByIDResponseTypedDict(TypedDict):
    id: int
    page_id: int
    title: str
    note: str
    start_time: datetime
    end_time: datetime
    created_at: datetime
    updated_at: datetime
    deleted_at: Nullable[datetime]
    component_id_list: NotRequired[List[int]]


class V4StatusPagesMaintenancesGetMaintenanceByIDResponse(BaseModel):
    id: Annotated[int, pydantic.Field(alias="ID")]

    page_id: Annotated[int, pydantic.Field(alias="PageID")]

    title: Annotated[str, pydantic.Field(alias="Title")]

    note: Annotated[str, pydantic.Field(alias="Note")]

    start_time: Annotated[datetime, pydantic.Field(alias="StartTime")]

    end_time: Annotated[datetime, pydantic.Field(alias="EndTime")]

    created_at: Annotated[datetime, pydantic.Field(alias="createdAt")]

    updated_at: Annotated[datetime, pydantic.Field(alias="updatedAt")]

    deleted_at: Annotated[Nullable[datetime], pydantic.Field(alias="deletedAt")]

    component_id_list: Annotated[
        Optional[List[int]], pydantic.Field(alias="ComponentIDList")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["ComponentIDList"]
        nullable_fields = ["deletedAt"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V4PauseResumeScheduleResponseTypedDict(TypedDict):
    action: str
    success: bool


class V4PauseResumeScheduleResponse(BaseModel):
    action: str

    success: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing_extensions import Annotated, TypedDict


class GlobalOncallReminderRulesDeleteGlobalOncallReminderRulesRequestTypedDict(
    TypedDict
):
    owner_id: str


class GlobalOncallReminderRulesDeleteGlobalOncallReminderRulesRequest(BaseModel):
    owner_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]


class GlobalOncallReminderRulesDeleteGlobalOncallReminderRulesResponseBodyTypedDict(
    TypedDict
):
    r"""There is no content to send for this request, but the headers may be useful."""


class GlobalOncallReminderRulesDeleteGlobalOncallReminderRulesResponseBody(BaseModel):
    r"""There is no content to send for this request, but the headers may be useful."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_globaleventruleinlist import (
    V3GlobalEventRulesGlobalEventRuleInList,
    V3GlobalEventRulesGlobalEventRuleInListTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import Callable, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GlobalEventRulesListGlobalEventRulesRequestTypedDict(TypedDict):
    owner_id: str
    page_size: NotRequired[int]
    page_number: NotRequired[int]
    filters_owner_id: NotRequired[List[str]]
    filters_search: NotRequired[str]


class GlobalEventRulesListGlobalEventRulesRequest(BaseModel):
    owner_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]

    page_size: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    page_number: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    filters_owner_id: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="filters.owner_id[]"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    filters_search: Annotated[
        Optional[str],
        pydantic.Field(alias="filters.search"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None


class GlobalEventRulesListGlobalEventRulesMetaTypedDict(TypedDict):
    total_count: int


class GlobalEventRulesListGlobalEventRulesMeta(BaseModel):
    total_count: int


class GlobalEventRulesListGlobalEventRulesResponseBodyTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V3GlobalEventRulesGlobalEventRuleInListTypedDict]
    meta: GlobalEventRulesListGlobalEventRulesMetaTypedDict


class GlobalEventRulesListGlobalEventRulesResponseBody(BaseModel):
    r"""The request has succeeded."""

    data: List[V3GlobalEventRulesGlobalEventRuleInList]

    meta: GlobalEventRulesListGlobalEventRulesMeta


class GlobalEventRulesListGlobalEventRulesResponseTypedDict(TypedDict):
    result: GlobalEventRulesListGlobalEventRulesResponseBodyTypedDict


class GlobalEventRulesListGlobalEventRulesResponse(BaseModel):
    next: Callable[[], Optional[GlobalEventRulesListGlobalEventRulesResponse]]

    result: GlobalEventRulesListGlobalEventRulesResponseBody



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


V3SLOSLOActionType = Literal[
    "SERVICE",
    "USER",
    "SQUAD",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class AbilitiesTypedDict(TypedDict):
    r"""A map of abilities granted to the user."""


class Abilities(BaseModel):
    r"""A map of abilities granted to the user."""


class CommonV3RBACEntityPermissionTypedDict(TypedDict):
    r"""Represents a permission granted to a user for a specific entity."""

    user_id: str
    r"""The ID of the user receiving the permission."""
    abilities: AbilitiesTypedDict
    r"""A map of abilities granted to the user."""


class CommonV3RBACEntityPermission(BaseModel):
    r"""Represents a permission granted to a user for a specific entity."""

    user_id: str
    r"""The ID of the user receiving the permission."""

    abilities: Abilities
    r"""A map of abilities granted to the user."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_routingrules_createorupdateroutingrulesrequest import (
    V3ServicesRoutingRulesCreateOrUpdateRoutingRulesRequest,
    V3ServicesRoutingRulesCreateOrUpdateRoutingRulesRequestTypedDict,
)
from .v3_services_routingrules_routingruleresponse import (
    V3ServicesRoutingRulesRoutingRuleResponse,
    V3ServicesRoutingRulesRoutingRuleResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import List
from typing_extensions import Annotated, TypedDict


class RoutingRulesCreateOrUpdateRoutingRulesRequestTypedDict(TypedDict):
    service_id: str
    v3_services_routing_rules_create_or_update_routing_rules_request: (
        V3ServicesRoutingRulesCreateOrUpdateRoutingRulesRequestTypedDict
    )


class RoutingRulesCreateOrUpdateRoutingRulesRequest(BaseModel):
    service_id: Annotated[
        str,
        pydantic.Field(alias="serviceID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_services_routing_rules_create_or_update_routing_rules_request: Annotated[
        V3ServicesRoutingRulesCreateOrUpdateRoutingRulesRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class RoutingRulesCreateOrUpdateRoutingRulesDataTypedDict(TypedDict):
    rules: List[V3ServicesRoutingRulesRoutingRuleResponseTypedDict]


class RoutingRulesCreateOrUpdateRoutingRulesData(BaseModel):
    rules: List[V3ServicesRoutingRulesRoutingRuleResponse]


class RoutingRulesCreateOrUpdateRoutingRulesResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: RoutingRulesCreateOrUpdateRoutingRulesDataTypedDict


class RoutingRulesCreateOrUpdateRoutingRulesResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: RoutingRulesCreateOrUpdateRoutingRulesData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3ExtensionsMSTeamsSquadCastServiceMappingTypedDict(TypedDict):
    r"""Maps a specific Squadcast service to a channel configuration."""

    squadcast_service_id: str
    r"""The MongoDB ObjectID of the Squadcast service."""
    squadcast_service_name: str
    r"""The name of the Squadcast service."""


class V3ExtensionsMSTeamsSquadCastServiceMapping(BaseModel):
    r"""Maps a specific Squadcast service to a channel configuration."""

    squadcast_service_id: str
    r"""The MongoDB ObjectID of the Squadcast service."""

    squadcast_service_name: str
    r"""The name of the Squadcast service."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_squads_squadmember import V4SquadsSquadMember, V4SquadsSquadMemberTypedDict
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4SquadsSquadResponseTypedDict(TypedDict):
    id: str
    org_id: str
    team_id: str
    name: str
    members: List[V4SquadsSquadMemberTypedDict]
    created_at: str
    updated_at: str
    created_by: NotRequired[str]
    updated_by: NotRequired[str]


class V4SquadsSquadResponse(BaseModel):
    id: str

    org_id: str

    team_id: str

    name: str

    members: List[V4SquadsSquadMember]

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]

    created_by: Annotated[Optional[str], pydantic.Field(alias="createdBy")] = None

    updated_by: Annotated[Optional[str], pydantic.Field(alias="updatedBy")] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_escalationpolicies_escalationpolicyresponse import (
    V3EscalationPoliciesEscalationPolicyResponse,
    V3EscalationPoliciesEscalationPolicyResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import Callable, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EscalationPoliciesGetEscalationPolicyByTeamRequestTypedDict(TypedDict):
    owner_id: str
    r"""here owner_id represents team_id, if  team_id is not provided, it will return escalation policies of all teams."""
    page_number: NotRequired[int]
    page_size: NotRequired[int]


class EscalationPoliciesGetEscalationPolicyByTeamRequest(BaseModel):
    owner_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""here owner_id represents team_id, if  team_id is not provided, it will return escalation policies of all teams."""

    page_number: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    page_size: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None


class EscalationPoliciesGetEscalationPolicyByTeamMetaTypedDict(TypedDict):
    total_count: int


class EscalationPoliciesGetEscalationPolicyByTeamMeta(BaseModel):
    total_count: int


class EscalationPoliciesGetEscalationPolicyByTeamResponseBodyTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V3EscalationPoliciesEscalationPolicyResponseTypedDict]
    meta: EscalationPoliciesGetEscalationPolicyByTeamMetaTypedDict


class EscalationPoliciesGetEscalationPolicyByTeamResponseBody(BaseModel):
    r"""The request has succeeded."""

    data: List[V3EscalationPoliciesEscalationPolicyResponse]

    meta: EscalationPoliciesGetEscalationPolicyByTeamMeta


class EscalationPoliciesGetEscalationPolicyByTeamResponseTypedDict(TypedDict):
    result: EscalationPoliciesGetEscalationPolicyByTeamResponseBodyTypedDict


class EscalationPoliciesGetEscalationPolicyByTeamResponse(BaseModel):
    next: Callable[[], Optional[EscalationPoliciesGetEscalationPolicyByTeamResponse]]

    result: EscalationPoliciesGetEscalationPolicyByTeamResponseBody



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import date
from jsonpath import JSONPath
from squadcast_sdk import errors, models, utils
from squadcast_sdk._hooks import HookContext
from squadcast_sdk.types import OptionalNullable, UNSET
from squadcast_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union


class AuditLogs(BaseSDK):
    def list(
        self,
        *,
        page_size: int,
        page_number: int,
        start_date: date,
        end_date: date,
        action: Optional[List[str]] = None,
        resource: Optional[List[str]] = None,
        actor: Optional[List[str]] = None,
        team: Optional[List[str]] = None,
        client: Optional[List[models.Client]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.AuditLogsListAuditLogsResponse]:
        r"""List all Audit Logs

        List all Audit Logs
        Returns array of audit logs for given team and filters

        :param page_size:
        :param page_number:
        :param start_date:
        :param end_date:
        :param action:
        :param resource:
        :param actor:
        :param team:
        :param client:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AuditLogsListAuditLogsRequest(
            page_size=page_size,
            page_number=page_number,
            start_date=start_date,
            end_date=end_date,
            action=action,
            resource=resource,
            actor=actor,
            team=team,
            client=client,
        )

        req = self._build_request(
            method="GET",
            path="/v3/audit-logs",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="AuditLogs_listAuditLogs",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.AuditLogsListAuditLogsResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page_number
            next_page = page + 1

            if not http_res.text:
                return None
            results = JSONPath("$.data").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.page_size
            if len(results[0]) < limit:
                return None

            return self.list(
                page_size=page_size,
                page_number=next_page,
                start_date=start_date,
                end_date=end_date,
                action=action,
                resource=resource,
                actor=actor,
                team=team,
                client=client,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AuditLogsListAuditLogsResponse(
                result=unmarshal_json_response(
                    models.V3AuditLogsListAuditLogsResponse, http_res
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        page_size: int,
        page_number: int,
        start_date: date,
        end_date: date,
        action: Optional[List[str]] = None,
        resource: Optional[List[str]] = None,
        actor: Optional[List[str]] = None,
        team: Optional[List[str]] = None,
        client: Optional[List[models.Client]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.AuditLogsListAuditLogsResponse]:
        r"""List all Audit Logs

        List all Audit Logs
        Returns array of audit logs for given team and filters

        :param page_size:
        :param page_number:
        :param start_date:
        :param end_date:
        :param action:
        :param resource:
        :param actor:
        :param team:
        :param client:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AuditLogsListAuditLogsRequest(
            page_size=page_size,
            page_number=page_number,
            start_date=start_date,
            end_date=end_date,
            action=action,
            resource=resource,
            actor=actor,
            team=team,
            client=client,
        )

        req = self._build_request_async(
            method="GET",
            path="/v3/audit-logs",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="AuditLogs_listAuditLogs",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.AuditLogsListAuditLogsResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page_number
            next_page = page + 1

            if not http_res.text:
                return None
            results = JSONPath("$.data").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.page_size
            if len(results[0]) < limit:
                return None

            return self.list(
                page_size=page_size,
                page_number=next_page,
                start_date=start_date,
                end_date=end_date,
                action=action,
                resource=resource,
                actor=actor,
                team=team,
                client=client,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AuditLogsListAuditLogsResponse(
                result=unmarshal_json_response(
                    models.V3AuditLogsListAuditLogsResponse, http_res
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def export(
        self,
        *,
        filters: Union[models.Filters, models.FiltersTypedDict],
        name: str,
        export_type: models.ExportType,
        description: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3AuditLogsExportAuditLogsResponseData:
        r"""Initiate an asynchronous export of audit logs based on the provided filters. The export file will be generated and available for download. Use 'Get details of Audit Logs export history by ID' API to retrieve the download URL.

        Export Audit logs
        Initiates export of audit logs based on provided filters

        :param filters:
        :param name:
        :param export_type:
        :param description:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3AuditLogsExportAuditLogsRequest(
            filters=utils.get_pydantic_model(filters, models.Filters),
            name=name,
            description=description,
            export_type=export_type,
        )

        req = self._build_request(
            method="POST",
            path="/v3/audit-logs/export",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.V3AuditLogsExportAuditLogsRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="AuditLogs_exportAuditLogs",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.V3AuditLogsExportAuditLogsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def export_async(
        self,
        *,
        filters: Union[models.Filters, models.FiltersTypedDict],
        name: str,
        export_type: models.ExportType,
        description: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3AuditLogsExportAuditLogsResponseData:
        r"""Initiate an asynchronous export of audit logs based on the provided filters. The export file will be generated and available for download. Use 'Get details of Audit Logs export history by ID' API to retrieve the download URL.

        Export Audit logs
        Initiates export of audit logs based on provided filters

        :param filters:
        :param name:
        :param export_type:
        :param description:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3AuditLogsExportAuditLogsRequest(
            filters=utils.get_pydantic_model(filters, models.Filters),
            name=name,
            description=description,
            export_type=export_type,
        )

        req = self._build_request_async(
            method="POST",
            path="/v3/audit-logs/export",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.V3AuditLogsExportAuditLogsRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="AuditLogs_exportAuditLogs",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.V3AuditLogsExportAuditLogsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def list_export_history(
        self,
        *,
        page_size: int,
        page_number: int,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.AuditLogsListAuditLogsExportHistoryResponse]:
        r"""List all Audit Logs export history

        List all Audit Logs export history
        Returns array of audit logs export history

        :param page_size:
        :param page_number:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AuditLogsListAuditLogsExportHistoryRequest(
            page_size=page_size,
            page_number=page_number,
        )

        req = self._build_request(
            method="GET",
            path="/v3/audit-logs/export/history",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="AuditLogs_listAuditLogsExportHistory",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.AuditLogsListAuditLogsExportHistoryResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page_number
            next_page = page + 1

            if not http_res.text:
                return None
            results = JSONPath("$.data").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.page_size
            if len(results[0]) < limit:
                return None

            return self.list_export_history(
                page_size=page_size,
                page_number=next_page,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AuditLogsListAuditLogsExportHistoryResponse(
                result=unmarshal_json_response(
                    models.V3AuditLogsListAuditLogsExportHistoryResponse, http_res
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def list_export_history_async(
        self,
        *,
        page_size: int,
        page_number: int,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.AuditLogsListAuditLogsExportHistoryResponse]:
        r"""List all Audit Logs export history

        List all Audit Logs export history
        Returns array of audit logs export history

        :param page_size:
        :param page_number:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AuditLogsListAuditLogsExportHistoryRequest(
            page_size=page_size,
            page_number=page_number,
        )

        req = self._build_request_async(
            method="GET",
            path="/v3/audit-logs/export/history",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="AuditLogs_listAuditLogsExportHistory",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.AuditLogsListAuditLogsExportHistoryResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page_number
            next_page = page + 1

            if not http_res.text:
                return None
            results = JSONPath("$.data").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.page_size
            if len(results[0]) < limit:
                return None

            return self.list_export_history(
                page_size=page_size,
                page_number=next_page,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AuditLogsListAuditLogsExportHistoryResponse(
                result=unmarshal_json_response(
                    models.V3AuditLogsListAuditLogsExportHistoryResponse, http_res
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def get_export_history_by_id(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3AuditLogsAuditLogsExportHistoryResponse:
        r"""Get details of Audit Logs export history by ID

        Get details of Audit Logs export history by ID
        Returns audit log export history details for the specified ID

        :param id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AuditLogsGetAuditLogsExportHistoryByIDRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/v3/audit-logs/export/history/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="AuditLogs_getAuditLogsExportHistoryById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.V3AuditLogsGetAuditLogExportHistoryByIDResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_export_history_by_id_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3AuditLogsAuditLogsExportHistoryResponse:
        r"""Get details of Audit Logs export history by ID

        Get details of Audit Logs export history by ID
        Returns audit log export history details for the specified ID

        :param id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AuditLogsGetAuditLogsExportHistoryByIDRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v3/audit-logs/export/history/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="AuditLogs_getAuditLogsExportHistoryById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.V3AuditLogsGetAuditLogExportHistoryByIDResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def get_by_id(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3AuditLogsAuditLogIDResponse:
        r"""Get audit log by ID

        Get audit log by ID
        Returns audit log details for the specified ID

        :param id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AuditLogsGetAuditLogByIDRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/v3/audit-logs/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="AuditLogs_getAuditLogById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.V3AuditLogsGetAuditLogByIDResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_by_id_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3AuditLogsAuditLogIDResponse:
        r"""Get audit log by ID

        Get audit log by ID
        Returns audit log details for the specified ID

        :param id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AuditLogsGetAuditLogByIDRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v3/audit-logs/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="AuditLogs_getAuditLogById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.V3AuditLogsGetAuditLogByIDResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


V3IncidentsExportFormat = Literal[
    "json",
    "csv",
]
r"""Defines the format of the export."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_workflows_getworkflowlogsresponse import (
    V3WorkflowsGetWorkflowLogsResponse,
    V3WorkflowsGetWorkflowLogsResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing import Callable, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WorkflowsGetWorkflowLogsRequestTypedDict(TypedDict):
    workflow_id: str
    page_size: NotRequired[int]
    page_number: NotRequired[int]


class WorkflowsGetWorkflowLogsRequest(BaseModel):
    workflow_id: Annotated[
        str,
        pydantic.Field(alias="workflowID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    page_size: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    page_number: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None


class WorkflowsGetWorkflowLogsResponseTypedDict(TypedDict):
    result: V3WorkflowsGetWorkflowLogsResponseTypedDict


class WorkflowsGetWorkflowLogsResponse(BaseModel):
    next: Callable[[], Optional[WorkflowsGetWorkflowLogsResponse]]

    result: V3WorkflowsGetWorkflowLogsResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3AnalyticsAnalyticsResponseTypedDict(TypedDict):
    mtta: float
    mttr: float


class V3AnalyticsAnalyticsResponse(BaseModel):
    mtta: float

    mttr: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_auditlogs_auditlogidresponse import (
    V3AuditLogsAuditLogIDResponse,
    V3AuditLogsAuditLogIDResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3AuditLogsGetAuditLogByIDResponseTypedDict(TypedDict):
    r"""Response model for getting audit log by ID"""

    data: V3AuditLogsAuditLogIDResponseTypedDict
    r"""Represents detailed audit log entry response"""


class V3AuditLogsGetAuditLogByIDResponse(BaseModel):
    r"""Response model for getting audit log by ID"""

    data: V3AuditLogsAuditLogIDResponse
    r"""Represents detailed audit log entry response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_issues_issue import (
    V4StatusPagesIssuesIssue,
    V4StatusPagesIssuesIssueTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import Annotated, TypedDict


class V4StatusPagesIssuesListIssuesResponseMetaTypedDict(TypedDict):
    total_count: int


class V4StatusPagesIssuesListIssuesResponseMeta(BaseModel):
    total_count: Annotated[int, pydantic.Field(alias="totalCount")]


class V4StatusPagesIssuesListIssuesResponseTypedDict(TypedDict):
    data: List[V4StatusPagesIssuesIssueTypedDict]
    meta: V4StatusPagesIssuesListIssuesResponseMetaTypedDict


class V4StatusPagesIssuesListIssuesResponse(BaseModel):
    data: List[V4StatusPagesIssuesIssue]

    meta: V4StatusPagesIssuesListIssuesResponseMeta



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_teams_teammember import V3TeamsTeamMember, V3TeamsTeamMemberTypedDict
from .v3_teams_teamrole import V3TeamsTeamRole, V3TeamsTeamRoleTypedDict
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class OrganizationTypedDict(TypedDict):
    id: str
    name: str
    slug: str


class Organization(BaseModel):
    id: str

    name: str

    slug: str


class V3TeamsTeamResponseTypedDict(TypedDict):
    id: str
    created_at: str
    updated_at: str
    created_by: str
    organization_id: str
    name: str
    description: str
    slug: str
    members: List[V3TeamsTeamMemberTypedDict]
    roles: List[V3TeamsTeamRoleTypedDict]
    default: bool
    organization: OrganizationTypedDict


class V3TeamsTeamResponse(BaseModel):
    id: str

    created_at: str

    updated_at: str

    created_by: str

    organization_id: str

    name: str

    description: str

    slug: str

    members: List[V3TeamsTeamMember]

    roles: List[V3TeamsTeamRole]

    default: bool

    organization: Organization



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3WorkflowsSlackCreateIncidentChannelName = Literal["slack_create_incident_channel",]


class V3WorkflowsSlackCreateIncidentChannelDataTypedDict(TypedDict):
    auto_name: bool
    channel_name: NotRequired[str]


class V3WorkflowsSlackCreateIncidentChannelData(BaseModel):
    auto_name: bool

    channel_name: Optional[str] = None


class V3WorkflowsSlackCreateIncidentChannelTypedDict(TypedDict):
    name: V3WorkflowsSlackCreateIncidentChannelName
    data: V3WorkflowsSlackCreateIncidentChannelDataTypedDict


class V3WorkflowsSlackCreateIncidentChannel(BaseModel):
    name: V3WorkflowsSlackCreateIncidentChannelName

    data: V3WorkflowsSlackCreateIncidentChannelData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_workflows_jiracreateticket import (
    V3WorkflowsJiraCreateTicket,
    V3WorkflowsJiraCreateTicketTypedDict,
)
from .v3_workflows_msteamscreatemeetinglink import (
    V3WorkflowsMsTeamsCreateMeetingLink,
    V3WorkflowsMsTeamsCreateMeetingLinkTypedDict,
)
from .v3_workflows_msteamsmessagechannel import (
    V3WorkflowsMsTeamsMessageChannel,
    V3WorkflowsMsTeamsMessageChannelTypedDict,
)
from .v3_workflows_msteamsmessageuser import (
    V3WorkflowsMsTeamsMessageUser,
    V3WorkflowsMsTeamsMessageUserTypedDict,
)
from .v3_workflows_runbookresponse import (
    V3WorkflowsRunbookResponse,
    V3WorkflowsRunbookResponseTypedDict,
)
from .v3_workflows_slackarchivechannel import (
    V3WorkflowsSlackArchiveChannel,
    V3WorkflowsSlackArchiveChannelTypedDict,
)
from .v3_workflows_slackcreateincidentchannel import (
    V3WorkflowsSlackCreateIncidentChannel,
    V3WorkflowsSlackCreateIncidentChannelTypedDict,
)
from .v3_workflows_slackmessagechannel import (
    V3WorkflowsSlackMessageChannel,
    V3WorkflowsSlackMessageChannelTypedDict,
)
from .v3_workflows_slackmessageuser import (
    V3WorkflowsSlackMessageUser,
    V3WorkflowsSlackMessageUserTypedDict,
)
from .v3_workflows_sqaddcommunicationchannel import (
    V3WorkflowsSqAddCommunicationChannel,
    V3WorkflowsSqAddCommunicationChannelTypedDict,
)
from .v3_workflows_sqaddincidentnote import (
    V3WorkflowsSqAddIncidentNote,
    V3WorkflowsSqAddIncidentNoteTypedDict,
)
from .v3_workflows_sqcreatestatuspageissue import (
    V3WorkflowsSqCreateStatusPageIssue,
    V3WorkflowsSqCreateStatusPageIssueTypedDict,
)
from .v3_workflows_sqmakehttpcall import (
    V3WorkflowsSqMakeHTTPCall,
    V3WorkflowsSqMakeHTTPCallTypedDict,
)
from .v3_workflows_sqmarkincidentsloaffecting import (
    V3WorkflowsSqMarkIncidentSLOAffecting,
    V3WorkflowsSqMarkIncidentSLOAffectingTypedDict,
)
from .v3_workflows_sqsendemail import (
    V3WorkflowsSqSendEmail,
    V3WorkflowsSqSendEmailTypedDict,
)
from .v3_workflows_sqtriggermanualwebhook import (
    V3WorkflowsSqTriggerManualWebhook,
    V3WorkflowsSqTriggerManualWebhookTypedDict,
)
from .v3_workflows_updateincidentpriority import (
    V3WorkflowsUpdateIncidentPriority,
    V3WorkflowsUpdateIncidentPriorityTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import Any, List, Literal, Union
from typing_extensions import TypeAliasType, TypedDict


V3WorkflowsActionResponseName = Literal["sq_attach_runbooks",]


class V3WorkflowsActionResponseDataTypedDict(TypedDict):
    runbooks: List[V3WorkflowsRunbookResponseTypedDict]


class V3WorkflowsActionResponseData(BaseModel):
    runbooks: List[V3WorkflowsRunbookResponse]


class V3WorkflowsActionResponseSqAttachRunbooksTypedDict(TypedDict):
    name: V3WorkflowsActionResponseName
    data: V3WorkflowsActionResponseDataTypedDict


class V3WorkflowsActionResponseSqAttachRunbooks(BaseModel):
    name: V3WorkflowsActionResponseName

    data: V3WorkflowsActionResponseData


V3WorkflowsActionResponseTypedDict = TypeAliasType(
    "V3WorkflowsActionResponseTypedDict",
    Union[
        V3WorkflowsSlackArchiveChannelTypedDict,
        V3WorkflowsMsTeamsCreateMeetingLinkTypedDict,
        V3WorkflowsSlackMessageChannelTypedDict,
        V3WorkflowsSlackMessageUserTypedDict,
        V3WorkflowsSqCreateStatusPageIssueTypedDict,
        V3WorkflowsSqAddIncidentNoteTypedDict,
        V3WorkflowsSqTriggerManualWebhookTypedDict,
        V3WorkflowsSqAddCommunicationChannelTypedDict,
        V3WorkflowsActionResponseSqAttachRunbooksTypedDict,
        V3WorkflowsUpdateIncidentPriorityTypedDict,
        V3WorkflowsSqMakeHTTPCallTypedDict,
        V3WorkflowsSlackCreateIncidentChannelTypedDict,
        V3WorkflowsJiraCreateTicketTypedDict,
        V3WorkflowsMsTeamsMessageChannelTypedDict,
        V3WorkflowsMsTeamsMessageUserTypedDict,
        V3WorkflowsSqSendEmailTypedDict,
        V3WorkflowsSqMarkIncidentSLOAffectingTypedDict,
        Any,
    ],
)


V3WorkflowsActionResponse = TypeAliasType(
    "V3WorkflowsActionResponse",
    Union[
        V3WorkflowsSlackArchiveChannel,
        V3WorkflowsMsTeamsCreateMeetingLink,
        V3WorkflowsSlackMessageChannel,
        V3WorkflowsSlackMessageUser,
        V3WorkflowsSqCreateStatusPageIssue,
        V3WorkflowsSqAddIncidentNote,
        V3WorkflowsSqTriggerManualWebhook,
        V3WorkflowsSqAddCommunicationChannel,
        V3WorkflowsActionResponseSqAttachRunbooks,
        V3WorkflowsUpdateIncidentPriority,
        V3WorkflowsSqMakeHTTPCall,
        V3WorkflowsSlackCreateIncidentChannel,
        V3WorkflowsJiraCreateTicket,
        V3WorkflowsMsTeamsMessageChannel,
        V3WorkflowsMsTeamsMessageUser,
        V3WorkflowsSqSendEmail,
        V3WorkflowsSqMarkIncidentSLOAffecting,
        Any,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_tag import V4Tag, V4TagTypedDict
import pydantic
from pydantic import model_serializer
from squadcast_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from typing import List, Literal
from typing_extensions import Annotated, TypedDict


V4ScheduleResponseOwnerType = Literal[
    "user",
    "squad",
]


class V4ScheduleResponseTypedDict(TypedDict):
    id: int
    name: str
    description: str
    org_id: str
    team_id: str
    time_zone: str
    paused: bool
    owner_id: str
    owner_type: V4ScheduleResponseOwnerType
    tags: Nullable[List[V4TagTypedDict]]
    created_at: str
    updated_at: str


class V4ScheduleResponse(BaseModel):
    id: int

    name: str

    description: str

    org_id: Annotated[str, pydantic.Field(alias="orgID")]

    team_id: Annotated[str, pydantic.Field(alias="teamID")]

    time_zone: Annotated[str, pydantic.Field(alias="timeZone")]

    paused: bool

    owner_id: Annotated[str, pydantic.Field(alias="ownerID")]

    owner_type: Annotated[
        V4ScheduleResponseOwnerType, pydantic.Field(alias="ownerType")
    ]

    tags: Nullable[List[V4Tag]]

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["tags"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


V3IncidentsPostmortemsPostmortemStatus = Literal[
    "in_progress",
    "under_review",
    "published",
]
r"""Represents the status of a postmortem."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_taggingrules_tagconfigobject import (
    V3ServicesTaggingRulesTagConfigObject,
    V3ServicesTaggingRulesTagConfigObjectTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3ServicesTaggingRulesTagsObjectTypedDict(TypedDict):
    severity: NotRequired[V3ServicesTaggingRulesTagConfigObjectTypedDict]


class V3ServicesTaggingRulesTagsObject(BaseModel):
    severity: Optional[V3ServicesTaggingRulesTagConfigObject] = None
